@@ -527,7 +527,7 @@ GNUNET_CHAT_stop (struct GNUNET_CHAT_Handle *handle);
  */
 enum GNUNET_GenericReturnValue
 GNUNET_CHAT_account_create (struct GNUNET_CHAT_Handle *handle,
-                            const char* name);
+                            const char *name);
 
 /**
  * Deletes an existing chat account of a given chat <i>handle</i> under a
@@ -539,7 +539,7 @@ GNUNET_CHAT_account_create (struct GNUNET_CHAT_Handle *handle,
  */
 enum GNUNET_GenericReturnValue
 GNUNET_CHAT_account_delete(struct GNUNET_CHAT_Handle *handle,
-                           const char* name);
+                           const char *name);
 
 /**
  * Iterates through the accounts of a given chat <i>handle</i> with a selected
@@ -555,6 +555,18 @@ GNUNET_CHAT_iterate_accounts (const struct GNUNET_CHAT_Handle *handle,
                               GNUNET_CHAT_AccountCallback callback,
                               void *cls);
 
+/*
+ * Searches for an existing chat account of a given chat <i>handle</i> with
+ * a unique <i>name</i>.
+ *
+ * @param[in] handle Chat handle
+ * @param[in] name Account name
+ * @return Found account by the handle or NULL
+ */
+struct GNUNET_CHAT_Account*
+GNUNET_CHAT_find_account (const struct GNUNET_CHAT_Handle *handle,
+                          const char *name);
+
 /**
  * Connects a chat <i>handle</i> to a selected chat <i>account</i>.
  *
@@ -933,10 +945,9 @@ GNUNET_CHAT_iterate_groups (struct GNUNET_CHAT_Handle *handle,
  * Leaves the private chat with a specific <i>contact</i> and frees the
  * regarding memory of the contact if there remains no common chat with it.
  *
- * @param[in,out] contact Cntact
- * @return #GNUNET_OK on success, #GNUNET_SYSERR on failure
+ * @param[in,out] contact Contact
  */
-enum GNUNET_GenericReturnValue
+void
 GNUNET_CHAT_contact_delete (struct GNUNET_CHAT_Contact *contact);
 
 /**
@@ -1578,8 +1589,8 @@ GNUNET_CHAT_message_iterate_tags (const struct GNUNET_CHAT_Message *message,
                                   void *cls);
 
 /**
- * Returns the amount of data from a given <i>message</i> if its kind 
- * is #GNUNET_CHAT_KIND_DATA, otherwise it returns NULL.
+ * Returns the amount of data from a given chat <i>message</i> if its 
+ * kind is #GNUNET_CHAT_KIND_DATA, otherwise it returns zero.
  *
  * @param[in] message Message
  * @return The amount of data or zero
@@ -1588,7 +1599,7 @@ uint64_t
 GNUNET_CHAT_message_available (const struct GNUNET_CHAT_Message *message);
 
 /**
- * Reads the data from a given <i>message</i> if its kind is
+ * Reads the data from a given chat <i>message</i> if its kind is
  * #GNUNET_CHAT_KIND_DATA into a specific <i>data</i> buffer
  * up to a selected <i>size</i>.
  *
@@ -1603,6 +1614,20 @@ GNUNET_CHAT_message_read (const struct GNUNET_CHAT_Message *message,
                           char *data,
                           uint64_t size);
 
+/**
+ * Feeds the data from a given chat <i>message</i> if its kind is
+ * #GNUNET_CHAT_KIND_DATA into a specific file descriptor (of pipe for 
+ * example).
+ *
+ * @param[in] message Message
+ * @param[in] fd File descriptor
+ * @return #GNUNET_OK on success, #GNUNET_NO if there's not enough data 
+ *         to read, otherwise #GNUNET_SYSERR on failure
+ */
+enum GNUNET_GenericReturnValue
+GNUNET_CHAT_message_feed (const struct GNUNET_CHAT_Message *message,
+                          int fd);
+
 /**
  * Returns the name of a given <i>file</i> handle.
  *
@@ -1843,6 +1868,27 @@ GNUNET_CHAT_discourse_get_id (const struct GNUNET_CHAT_Discourse *discourse);
 enum GNUNET_GenericReturnValue
 GNUNET_CHAT_discourse_is_open (const struct GNUNET_CHAT_Discourse *discourse);
 
+/**
+ * Sets a custom <i>user pointer</i> to a given chat <i>discourse</i> so it can
+ * be accessed in chat discourse related callbacks.
+ *
+ * @param[in,out] discourse Chat discourse
+ * @param[in] user_pointer Custom user pointer
+ */
+void
+GNUNET_CHAT_discourse_set_user_pointer (struct GNUNET_CHAT_Discourse *discourse,
+                                        void *user_pointer);
+
+/**
+ * Returns the custom user pointer of a given chat <i>discourse</i> or NULL if it
+ * was not set any.
+ *
+ * @param[in] discourse Chat discourse
+ * @return Custom user pointer or NULL
+ */
+void*
+GNUNET_CHAT_discourse_get_user_pointer (const struct GNUNET_CHAT_Discourse *discourse);
+
 /**
  * Closes a given chat <i>discourse</i> to stop receiving any data messages
  * from the specific discourse.
@@ -1866,6 +1912,16 @@ GNUNET_CHAT_discourse_write (struct GNUNET_CHAT_Discourse *discourse,
                              const char *data,
                              uint64_t size);
 
+/**
+ * Returns a file descriptor of a pipe to write data via IPC
+ * into the given chat <i>discourse</i>.
+ *
+ * @param[in] discourse Chat discourse
+ * @return File descriptor of pipe end or #GNUNET_SYSERR on failure
+ */
+int
+GNUNET_CHAT_discourse_get_fd (const struct GNUNET_CHAT_Discourse *discourse);
+
 /**
  * Iterates through the subscribed chat contacts of a given chat <i>discourse</i> 
  * with a selected callback and custom closure.
