@@ -119,12 +119,13 @@ context_destroy (struct GNUNET_CHAT_Context *context)
 {
   GNUNET_assert(
     (context) &&
-		(context->timestamps) &&
+    (context->timestamps) &&
     (context->dependencies) &&
-		(context->messages) &&
+    (context->messages) &&
     (context->taggings) &&
-		(context->invites) &&
-		(context->files)
+    (context->invites) &&
+    (context->files) &&
+    (context->discourses)
   );
 
   if (context->request_task)
@@ -150,6 +151,10 @@ context_destroy (struct GNUNET_CHAT_Context *context)
     context->invites, it_destroy_context_invites, NULL
   );
 
+  GNUNET_CONTAINER_multishortmap_iterate(
+    context->discourses, it_destroy_context_discourses, NULL
+  );
+
   GNUNET_CONTAINER_multishortmap_destroy(context->member_pointers);
 
   GNUNET_CONTAINER_multishortmap_destroy(context->timestamps);
@@ -184,7 +189,7 @@ context_request_message (struct GNUNET_CHAT_Context* context,
     return;
 
   context->request_task = GNUNET_SCHEDULER_add_with_priority(
-    GNUNET_SCHEDULER_PRIORITY_IDLE,
+    GNUNET_SCHEDULER_PRIORITY_BACKGROUND,
     cb_context_request_messages,
     context
   );
@@ -200,6 +205,14 @@ context_update_room (struct GNUNET_CHAT_Context *context,
   if (room == context->room)
     return;
 
+  GNUNET_assert(
+    (context->timestamps) &&
+    (context->messages) &&
+    (context->requests) &&
+    (context->invites) &&
+    (context->discourses)
+  );
+
   GNUNET_CONTAINER_multishortmap_iterate(
     context->timestamps, it_destroy_context_timestamps, NULL
   );
@@ -212,6 +225,10 @@ context_update_room (struct GNUNET_CHAT_Context *context,
     context->invites, it_destroy_context_invites, NULL
   );
 
+  GNUNET_CONTAINER_multishortmap_iterate(
+    context->discourses, it_destroy_context_discourses, NULL
+  );
+
   GNUNET_CONTAINER_multishortmap_destroy(context->timestamps);
   context->timestamps = GNUNET_CONTAINER_multishortmap_create(
     initial_map_size_of_room, GNUNET_NO);
@@ -221,6 +238,10 @@ context_update_room (struct GNUNET_CHAT_Context *context,
   GNUNET_CONTAINER_multihashmap_clear(context->invites);
   GNUNET_CONTAINER_multihashmap_clear(context->files);
 
+  GNUNET_CONTAINER_multishortmap_destroy(context->discourses);
+  context->discourses = GNUNET_CONTAINER_multishortmap_create(
+    initial_map_size_of_room, GNUNET_NO);
+
   if (context->room)
     context_delete(context, GNUNET_YES);
 
@@ -255,7 +276,8 @@ context_update_nick (struct GNUNET_CHAT_Context *context,
     NULL,
     context,
     GNUNET_CHAT_FLAG_UPDATE_CONTEXT,
-    NULL
+    NULL,
+    GNUNET_NO
   );
 }
 
@@ -362,7 +384,7 @@ context_delete_message (struct GNUNET_CHAT_Context *context,
     }
     case GNUNET_MESSENGER_KIND_TAG:
     {
-      struct GNUNET_CHAT_Tagging *tagging = GNUNET_CONTAINER_multihashmap_get(
+      struct GNUNET_CHAT_InternalTagging *tagging = GNUNET_CONTAINER_multihashmap_get(
         context->taggings,
         &(message->msg->body.tag.hash)
       );
@@ -370,7 +392,7 @@ context_delete_message (struct GNUNET_CHAT_Context *context,
       if (!tagging)
         break;
 
-      tagging_remove(tagging, message);
+      internal_tagging_remove(tagging, message);
       break;
     }
     default:
