@@ -31,10 +31,11 @@
 #include "gnunet_chat_invitation.h"
 #include "gnunet_chat_lobby.h"
 #include "gnunet_chat_message.h"
-#include "gnunet_chat_tagging.h"
 #include "gnunet_chat_ticket.h"
 #include "gnunet_chat_util.h"
+
 #include "internal/gnunet_chat_accounts.h"
+#include "internal/gnunet_chat_tagging.h"
 
 #include <gnunet/gnunet_arm_service.h>
 #include <gnunet/gnunet_common.h>
@@ -291,7 +292,8 @@ on_handle_refresh (void *cls)
     NULL,
     NULL,
     GNUNET_CHAT_FLAG_REFRESH,
-    NULL
+    NULL,
+    GNUNET_YES
   );
 }
 
@@ -321,7 +323,8 @@ on_handle_gnunet_identity (void *cls,
     if (ego != accounts->account->ego)
       goto check_matching_name;
 
-    if (name)
+    if ((name) && ((!(accounts->account->name)) ||
+        (0 != strcmp(accounts->account->name, name))))
     {
       util_set_name_field(name, &(accounts->account->name));
 
@@ -330,10 +333,11 @@ on_handle_gnunet_identity (void *cls,
         accounts->account,
         NULL,
         GNUNET_CHAT_FLAG_UPDATE_ACCOUNT,
-        NULL
+        NULL,
+        GNUNET_YES
       );
     }
-    else if (!(accounts->op))
+    else if ((!name) && (!(accounts->op)))
     {
       if (handle->current == accounts->account)
 	      handle_disconnect(handle);
@@ -341,6 +345,8 @@ on_handle_gnunet_identity (void *cls,
       account_destroy(accounts->account);
       internal_accounts_destroy(accounts);
     }
+    else if (!name)
+      account_update_ego(accounts->account, handle, NULL);
 
     goto send_refresh;
 
@@ -371,9 +377,8 @@ send_refresh:
   if (handle->refresh)
     return;
   
-  handle->refresh = GNUNET_SCHEDULER_add_at_with_priority(
-    GNUNET_TIME_absolute_get(),
-    GNUNET_SCHEDULER_PRIORITY_IDLE,
+  handle->refresh = GNUNET_SCHEDULER_add_with_priority(
+    GNUNET_SCHEDULER_PRIORITY_BACKGROUND,
     on_handle_refresh,
     handle
   );
@@ -407,7 +412,8 @@ cb_account_creation (void *cls,
     accounts->account,
     NULL,
     GNUNET_CHAT_FLAG_WARNING,
-    GNUNET_ErrorCode_get_hint(ec)
+    GNUNET_ErrorCode_get_hint(ec),
+    GNUNET_YES
   );
 }
 
@@ -434,7 +440,8 @@ cb_account_deletion (void *cls,
       accounts->account,
       NULL,
       GNUNET_CHAT_FLAG_WARNING,
-      GNUNET_ErrorCode_get_hint(ec)
+      GNUNET_ErrorCode_get_hint(ec),
+      GNUNET_YES
     );
   else
   {
@@ -443,7 +450,8 @@ cb_account_deletion (void *cls,
       accounts->account,
       NULL,
       GNUNET_CHAT_FLAG_DELETE_ACCOUNT,
-      NULL
+      NULL,
+      GNUNET_YES
     );
 
     account_delete(accounts->account);
@@ -475,7 +483,8 @@ cb_account_rename (void *cls,
     accounts->account,
     NULL,
     GNUNET_CHAT_FLAG_WARNING,
-    GNUNET_ErrorCode_get_hint(ec)
+    GNUNET_ErrorCode_get_hint(ec),
+    GNUNET_YES
   );
 }
 
@@ -499,7 +508,8 @@ cb_lobby_deletion (void *cls,
       accounts->account,
       NULL,
       GNUNET_CHAT_FLAG_WARNING,
-      GNUNET_ErrorCode_get_hint(ec)
+      GNUNET_ErrorCode_get_hint(ec),
+      GNUNET_YES
     );
 
   internal_accounts_destroy(accounts);
@@ -678,11 +688,35 @@ it_context_iterate_dependencies(void *cls,
 
   if ((message) && (!message->task))
     message->task = GNUNET_SCHEDULER_add_now(
-      on_handle_message_callback, message);
+      on_handle_message_callback, message
+    );
 
   return GNUNET_YES;
 }
 
+void
+on_handle_internal_message_callback(void *cls)
+{
+  struct GNUNET_CHAT_InternalMessages *internal = cls;
+
+  GNUNET_assert(
+    (internal) &&
+    (internal->chat) &&
+    (internal->msg) &&
+    (internal->task)
+  );
+
+  internal->task = NULL;
+
+  struct GNUNET_CHAT_Handle *handle = internal->chat;
+  struct GNUNET_CHAT_Context *context = internal->msg->context;
+
+  if (!(handle->msg_cb))
+    return;
+
+  handle->msg_cb(handle->msg_cls, context, internal->msg);
+}
+
 void
 on_handle_message_callback(void *cls)
 {
@@ -792,12 +826,12 @@ on_handle_message_callback(void *cls)
     }
     case GNUNET_MESSENGER_KIND_TAG:
     {
-      struct GNUNET_CHAT_Tagging *tagging = GNUNET_CONTAINER_multihashmap_get(
+      struct GNUNET_CHAT_InternalTagging *tagging = GNUNET_CONTAINER_multihashmap_get(
         context->taggings, &(message->msg->body.tag.hash));
       
       if (!tagging)
       {
-        tagging = tagging_create();
+        tagging = internal_tagging_create();
 
         if (!tagging)
           break;
@@ -806,12 +840,12 @@ on_handle_message_callback(void *cls)
             context->taggings, &(message->msg->body.tag.hash), tagging,
             GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_FAST))
         {
-          tagging_destroy(tagging);
+          internal_tagging_destroy(tagging);
           break;
         }
       }
 
-      tagging_add(tagging, message);
+      internal_tagging_add(tagging, message);
       break;
     }
     default:
@@ -910,6 +944,8 @@ skip_msg_handing:
         }
       }
 
+      enum GNUNET_GenericReturnValue subscribtion_update = GNUNET_NO;
+
       if (GNUNET_MESSENGER_FLAG_SUBSCRIPTION_UNSUBSCRIBE & message->msg->body.subscribe.flags)
         discourse_unsubscribe(
           discourse,
@@ -918,12 +954,15 @@ skip_msg_handing:
           GNUNET_TIME_relative_ntoh(message->msg->body.subscribe.time)
         );
       else
-        discourse_subscribe(
+        subscribtion_update = discourse_subscribe(
           discourse,
           contact,
           GNUNET_TIME_absolute_ntoh(message->msg->header.timestamp),
           GNUNET_TIME_relative_ntoh(message->msg->body.subscribe.time)
         );
+      
+      if (GNUNET_YES == subscribtion_update)
+        message->flags |= GNUNET_MESSENGER_FLAG_UPDATE;
 
       break;
     }
