@@ -32,10 +32,12 @@
 #include <gnunet/gnunet_scheduler_lib.h>
 #include <gnunet/gnunet_time_lib.h>
 #include <gnunet/gnunet_util_lib.h>
+
 #include <libgen.h>
 #include <stdint.h>
 #include <string.h>
 #include <strings.h>
+#include <unistd.h>
 
 #define _(String) ((const char*) String)
 
@@ -48,10 +50,11 @@
 #include "gnunet_chat_invitation.h"
 #include "gnunet_chat_lobby.h"
 #include "gnunet_chat_message.h"
-#include "gnunet_chat_tagging.h"
 #include "gnunet_chat_ticket.h"
 #include "gnunet_chat_util.h"
 
+#include "internal/gnunet_chat_tagging.h"
+
 #include "gnunet_chat_lib_intern.c"
 
 #define GNUNET_CHAT_VERSION_ASSERT() {\
@@ -89,8 +92,7 @@ GNUNET_CHAT_stop (struct GNUNET_CHAT_Handle *handle)
   if ((!handle) || (handle->destruction))
     return;
 
-  handle->destruction = GNUNET_SCHEDULER_add_at_with_priority(
-    GNUNET_TIME_absolute_get(),
+  handle->destruction = GNUNET_SCHEDULER_add_with_priority(
     GNUNET_SCHEDULER_PRIORITY_URGENT,
     task_handle_destruction,
     handle
@@ -160,6 +162,32 @@ GNUNET_CHAT_iterate_accounts (const struct GNUNET_CHAT_Handle *handle,
 }
 
 
+struct GNUNET_CHAT_Account*
+GNUNET_CHAT_find_account (const struct GNUNET_CHAT_Handle *handle,
+                          const char *name)
+{
+  GNUNET_CHAT_VERSION_ASSERT();
+
+  if ((!handle) || (handle->destruction))
+    return NULL;
+
+  struct GNUNET_CHAT_InternalAccounts *accounts = handle->accounts_head;
+  while (accounts)
+  {
+    if ((!(accounts->account)) || (accounts->op))
+      goto skip_account;
+
+    if (0 == strcmp(accounts->account->name, name))
+      return accounts->account;
+
+  skip_account:
+    accounts = accounts->next;
+  }
+
+  return NULL;
+}
+
+
 void
 GNUNET_CHAT_connect (struct GNUNET_CHAT_Handle *handle,
                      const struct GNUNET_CHAT_Account *account)
@@ -628,29 +656,13 @@ GNUNET_CHAT_lobby_close (struct GNUNET_CHAT_Lobby *lobby)
 {
   GNUNET_CHAT_VERSION_ASSERT();
 
-  if (!lobby)
+  if ((!lobby) || (lobby->destruction))
     return;
 
-  struct GNUNET_CHAT_InternalLobbies *lobbies = lobby->handle->lobbies_head;
-
-  while (lobbies)
-  {
-    if (lobbies->lobby == lobby)
-    {
-      GNUNET_CONTAINER_DLL_remove(
-        lobby->handle->lobbies_head,
-        lobby->handle->lobbies_tail,
-        lobbies
-      );
-
-      GNUNET_free(lobbies);
-      break;
-    }
-
-    lobbies = lobbies->next;
-  }
-
-  lobby_destroy(lobby);
+  lobby->destruction = GNUNET_SCHEDULER_add_now(
+    task_lobby_destruction,
+    lobby
+  );
 }
 
 
@@ -852,6 +864,29 @@ GNUNET_CHAT_iterate_files (struct GNUNET_CHAT_Handle *handle,
 }
 
 
+int
+GNUNET_CHAT_context_iterate_discourses (struct GNUNET_CHAT_Context *context,
+                                        GNUNET_CHAT_DiscourseCallback callback,
+                                        void *cls)
+{
+  GNUNET_CHAT_VERSION_ASSERT();
+
+  if ((!context) || (!(context->discourses)))
+    return GNUNET_SYSERR;
+
+  struct GNUNET_CHAT_ContextIterateDiscourses it;
+  it.context = context;
+  it.cb = callback;
+  it.cls = cls;
+
+  return GNUNET_CONTAINER_multishortmap_iterate(
+    context->discourses,
+    it_context_iterate_discourses,
+    &it
+  );
+}
+
+
 void
 GNUNET_CHAT_set_user_pointer (struct GNUNET_CHAT_Handle *handle,
 			                        void *user_pointer)
@@ -1059,34 +1094,21 @@ GNUNET_CHAT_iterate_groups (struct GNUNET_CHAT_Handle *handle,
 }
 
 
-enum GNUNET_GenericReturnValue
+void
 GNUNET_CHAT_contact_delete (struct GNUNET_CHAT_Contact *contact)
 {
   GNUNET_CHAT_VERSION_ASSERT();
 
-  if (!contact)
-    return GNUNET_SYSERR;
+  if ((!contact) || (contact->destruction))
+    return;
 
-  struct GNUNET_ShortHashCode shorthash;
-  util_shorthash_from_member(contact->member, &shorthash);
+  if (contact->context)
+    contact->context->deleted = GNUNET_YES;
 
-  GNUNET_CONTAINER_multishortmap_remove(
-    contact->handle->contacts, &shorthash, contact
+  contact->destruction = GNUNET_SCHEDULER_add_now(
+    task_contact_destruction,
+    contact
   );
-
-  const struct GNUNET_HashCode *key = GNUNET_MESSENGER_room_get_key(
-    contact->context->room
-  );
-
-  GNUNET_CONTAINER_multihashmap_remove(
-    contact->handle->contexts, key, contact->context
-  );
-
-  context_delete(contact->context, GNUNET_YES);
-  
-  context_destroy(contact->context);
-  contact_destroy(contact);
-  return GNUNET_OK;
 }
 
 
@@ -2170,13 +2192,13 @@ GNUNET_CHAT_message_is_tagged (const struct GNUNET_CHAT_Message *message,
   if ((!message) || (!(message->context)))
     return GNUNET_SYSERR;
 
-  const struct GNUNET_CHAT_Tagging *tagging = GNUNET_CONTAINER_multihashmap_get(
+  const struct GNUNET_CHAT_InternalTagging *tagging = GNUNET_CONTAINER_multihashmap_get(
     message->context->taggings, &(message->hash));
   
   if (!tagging)
     return GNUNET_NO;
 
-  if (tagging_iterate(tagging, GNUNET_NO, tag, NULL, NULL) > 0)
+  if (internal_tagging_iterate(tagging, GNUNET_NO, tag, NULL, NULL) > 0)
     return GNUNET_YES;
   else
     return GNUNET_NO;
@@ -2291,7 +2313,7 @@ GNUNET_CHAT_message_get_discourse (const struct GNUNET_CHAT_Message *message)
   GNUNET_CHAT_VERSION_ASSERT();
 
   if ((!message) || (GNUNET_YES != message_has_msg(message)) ||
-      (!(message->context)))
+      (!(message->context)) || (!(message->context->discourses)))
     return NULL;
   
   struct GNUNET_CHAT_Discourse *discourse;
@@ -2360,13 +2382,13 @@ GNUNET_CHAT_message_iterate_tags (const struct GNUNET_CHAT_Message *message,
   if ((!message) || (!(message->context)))
     return GNUNET_SYSERR;
 
-  const struct GNUNET_CHAT_Tagging *tagging = GNUNET_CONTAINER_multihashmap_get(
+  const struct GNUNET_CHAT_InternalTagging *tagging = GNUNET_CONTAINER_multihashmap_get(
     message->context->taggings, &(message->hash));
   
   if (!tagging)
     return 0;
 
-  return tagging_iterate(tagging, GNUNET_YES, NULL, callback, cls);
+  return internal_tagging_iterate(tagging, GNUNET_YES, NULL, callback, cls);
 }
 
 
@@ -2413,6 +2435,37 @@ GNUNET_CHAT_message_read (const struct GNUNET_CHAT_Message *message,
 }
 
 
+enum GNUNET_GenericReturnValue
+GNUNET_CHAT_message_feed (const struct GNUNET_CHAT_Message *message,
+                          int fd)
+{
+  GNUNET_CHAT_VERSION_ASSERT();
+
+  if ((!message) || (GNUNET_YES != message_has_msg(message)) ||
+      (fd == -1))
+    return GNUNET_SYSERR;
+
+  if (GNUNET_MESSENGER_KIND_TALK != message->msg->header.kind)
+    return GNUNET_SYSERR;
+
+  if (!(message->msg->body.talk.length))
+    return GNUNET_NO;
+
+  const ssize_t written = write(
+    fd,
+    message->msg->body.talk.data,
+    message->msg->body.talk.length
+  );
+
+  if (-1 == written)
+    return GNUNET_SYSERR;
+  else if (written != message->msg->body.talk.length)
+    return GNUNET_NO;
+  else
+    return GNUNET_OK;
+}
+
+
 const char*
 GNUNET_CHAT_file_get_name (const struct GNUNET_CHAT_File *file)
 {
@@ -2861,13 +2914,13 @@ GNUNET_CHAT_invitation_is_rejected (const struct GNUNET_CHAT_Invitation *invitat
   if (!invitation)
     return GNUNET_NO;
 
-  const struct GNUNET_CHAT_Tagging *tagging = GNUNET_CONTAINER_multihashmap_get(
+  const struct GNUNET_CHAT_InternalTagging *tagging = GNUNET_CONTAINER_multihashmap_get(
     invitation->context->taggings, &(invitation->hash));
   
   if (!tagging)
     return GNUNET_NO;
   
-  if (tagging_iterate(tagging, GNUNET_NO, NULL, NULL, NULL) > 0)
+  if (internal_tagging_iterate(tagging, GNUNET_NO, NULL, NULL, NULL) > 0)
     return GNUNET_YES;
   else
     return GNUNET_NO;
@@ -2908,6 +2961,31 @@ GNUNET_CHAT_discourse_is_open (const struct GNUNET_CHAT_Discourse *discourse)
 }
 
 
+void
+GNUNET_CHAT_discourse_set_user_pointer (struct GNUNET_CHAT_Discourse *discourse,
+                                        void *user_pointer)
+{
+  GNUNET_CHAT_VERSION_ASSERT();
+
+  if (!discourse)
+    return;
+
+  discourse->user_pointer = user_pointer;
+}
+
+
+void*
+GNUNET_CHAT_discourse_get_user_pointer (const struct GNUNET_CHAT_Discourse *discourse)
+{
+  GNUNET_CHAT_VERSION_ASSERT();
+
+  if (!discourse)
+    return NULL;
+
+  return discourse->user_pointer;
+}
+
+
 void
 GNUNET_CHAT_discourse_close (struct GNUNET_CHAT_Discourse *discourse)
 {
@@ -2987,6 +3065,18 @@ GNUNET_CHAT_discourse_write (struct GNUNET_CHAT_Discourse *discourse,
 }
 
 
+int
+GNUNET_CHAT_discourse_get_fd (const struct GNUNET_CHAT_Discourse *discourse)
+{
+  GNUNET_CHAT_VERSION_ASSERT();
+
+  if (! discourse)
+    return GNUNET_SYSERR;
+
+  return discourse->pipe[1];
+}
+
+
 int
 GNUNET_CHAT_discourse_iterate_contacts (const struct GNUNET_CHAT_Discourse *discourse,
                                         GNUNET_CHAT_DiscourseContactCallback callback,
