@@ -53,12 +53,9 @@ task_handle_destruction (void *cls)
 
   if (accounts)
   {
-    handle->destruction = GNUNET_SCHEDULER_add_at_with_priority(
-      GNUNET_TIME_absolute_add(
-          GNUNET_TIME_absolute_get(),
-          GNUNET_TIME_relative_get_millisecond_()
-      ),
-      GNUNET_SCHEDULER_PRIORITY_IDLE,
+    handle->destruction = GNUNET_SCHEDULER_add_delayed_with_priority(
+      GNUNET_TIME_relative_get_millisecond_(),
+      GNUNET_SCHEDULER_PRIORITY_BACKGROUND,
       task_handle_destruction,
       handle
     );
@@ -282,6 +279,66 @@ it_room_find_contact (void *cls,
   return GNUNET_NO;
 }
 
+void
+task_lobby_destruction (void *cls)
+{
+  GNUNET_assert(cls);
+
+  struct GNUNET_CHAT_Lobby *lobby = (struct GNUNET_CHAT_Lobby*) cls;
+  struct GNUNET_CHAT_InternalLobbies *lobbies = lobby->handle->lobbies_head;
+
+  while (lobbies)
+  {
+    if (lobbies->lobby == lobby)
+    {
+      GNUNET_CONTAINER_DLL_remove(
+        lobby->handle->lobbies_head,
+        lobby->handle->lobbies_tail,
+        lobbies
+      );
+
+      GNUNET_free(lobbies);
+      break;
+    }
+
+    lobbies = lobbies->next;
+  }
+
+  lobby->destruction = NULL;
+
+  lobby_destroy(lobby);
+}
+
+void
+task_contact_destruction (void *cls)
+{
+  GNUNET_assert(cls);
+
+  struct GNUNET_CHAT_Contact *contact = (struct GNUNET_CHAT_Contact*) cls;
+  struct GNUNET_ShortHashCode shorthash;
+
+  util_shorthash_from_member(contact->member, &shorthash);
+
+  GNUNET_CONTAINER_multishortmap_remove(
+    contact->handle->contacts, &shorthash, contact
+  );
+
+  const struct GNUNET_HashCode *key = GNUNET_MESSENGER_room_get_key(
+    contact->context->room
+  );
+
+  GNUNET_CONTAINER_multihashmap_remove(
+    contact->handle->contexts, key, contact->context
+  );
+
+  context_delete(contact->context, GNUNET_YES);
+  context_destroy(contact->context);
+
+  contact->destruction = NULL;
+
+  contact_destroy(contact);
+}
+
 void
 task_group_destruction (void *cls)
 {
@@ -389,6 +446,29 @@ it_context_iterate_files (void *cls,
   return it->cb(it->cls, it->context, file);
 }
 
+struct GNUNET_CHAT_ContextIterateDiscourses
+{
+  struct GNUNET_CHAT_Context *context;
+  GNUNET_CHAT_DiscourseCallback cb;
+  void *cls;
+};
+
+enum GNUNET_GenericReturnValue
+it_context_iterate_discourses (void *cls,
+                               GNUNET_UNUSED const struct GNUNET_ShortHashCode *key,
+                               void *value)
+{
+  GNUNET_assert((cls) && (value));
+
+  struct GNUNET_CHAT_ContextIterateDiscourses *it = cls;
+  struct GNUNET_CHAT_Discourse *discourse = value;
+
+  if (!(it->cb))
+    return GNUNET_YES;
+
+  return it->cb(it->cls, it->context, discourse);
+}
+
 struct GNUNET_CHAT_MessageIterateReadReceipts
 {
   const struct GNUNET_CHAT_Message *message;
@@ -469,7 +549,8 @@ cont_update_attribute_with_status (void *cls,
       account,
       NULL,
       GNUNET_CHAT_KIND_WARNING,
-      emsg
+      emsg,
+      GNUNET_YES
     );
   else
     handle_send_internal_message(
@@ -477,7 +558,8 @@ cont_update_attribute_with_status (void *cls,
       account,
       NULL,
       GNUNET_CHAT_FLAG_ATTRIBUTES,
-      attribute_name
+      attribute_name,
+      GNUNET_YES
     );
   
   internal_attributes_destroy(attributes);
@@ -544,7 +626,8 @@ cb_task_error_iterate_attribute (void *cls)
     attributes->account,
     NULL,
     GNUNET_CHAT_FLAG_WARNING,
-    "Attribute iteration failed!"
+    "Attribute iteration failed!",
+    GNUNET_YES
   );
 
   cb_task_finish_iterate_attribute(cls);
@@ -869,7 +952,8 @@ cb_task_error_iterate_ticket (void *cls)
     NULL,
     NULL,
     GNUNET_CHAT_FLAG_WARNING,
-    "Ticket iteration failed!"
+    "Ticket iteration failed!",
+    GNUNET_YES
   );
 
   cb_task_finish_iterate_ticket(cls);
@@ -896,7 +980,8 @@ cont_revoke_ticket (void *cls,
       NULL,
       NULL,
       GNUNET_CHAT_FLAG_WARNING,
-      emsg
+      emsg,
+      GNUNET_YES
     );
   else
     handle_send_internal_message(
@@ -904,7 +989,8 @@ cont_revoke_ticket (void *cls,
       NULL,
       NULL,
       GNUNET_CHAT_FLAG_SHARE_ATTRIBUTES,
-      NULL
+      NULL,
+      GNUNET_NO
     );
 
   internal_tickets_destroy(tickets);
